package de.javabeginners.plugin.notice.ui.elements;

import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * A dialog which is opened, when a user wants to create an new notice-tab. A
 * tab provides ether a list, a table or a plain text area, to display and store
 * arbitrary user data.
 */
public class NewTabDialog extends TabDialog {

	private ComboViewer typeCombo, numCombo;
	private Label numLabel;

	/**
	 * The type of the tab, i.e. <code>Text, Liste, Tabelle</code>.
	 */
	private String tabTypeStr;

	/**
	 * String representation of the number of columns of a table tab.
	 */
	private String colCountStr;

	/**
	 * Creates a dialog which is opened, when a user wants to create an new
	 * notice-tab.
	 * 
	 * @param parent
	 *            <code>Shell</code>, the shell of the workspace site.
	 */
	public NewTabDialog(Shell parent) {
		super(parent);
	}

	@Override
	public void create() {
		super.create();
		setTitle("New Tab");
		setMessage("Set a tab title and select an input type",
				IMessageProvider.INFORMATION);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		super.createDialogArea(parent);
		// initialisations
		Label typeLabel = new Label(container, SWT.NONE);
		typeCombo = new ComboViewer(container, SWT.NONE);
		numLabel = new Label(container, SWT.NONE);
		numCombo = new ComboViewer(container, SWT.NONE);

		// container with 4 columns
		GridLayout layout = new GridLayout(4, false);
		container.setLayout(layout);

		// tab title
		GridData tabTitleTextData = new GridData();
		tabTitleTextData.grabExcessHorizontalSpace = true;
		tabTitleTextData.horizontalAlignment = GridData.FILL;
		tabTitleTextData.horizontalSpan = 3;
		tabTitleText.setLayoutData(tabTitleTextData);

		// tab type combo
		typeLabel.setText("Tab Type");
		GridData typeComboData = new GridData();
		typeComboData.grabExcessHorizontalSpace = true;
		typeComboData.horizontalSpan = 1;
		typeComboData.horizontalAlignment = GridData.FILL;
		typeCombo.getCombo().setLayoutData(typeComboData);
		typeCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (!(element instanceof TabType)) {
					throw new IllegalArgumentException();
				}
				TabType tt = (TabType) element;
				return tt.getType();
			}
		});
		typeCombo.setContentProvider(ArrayContentProvider.getInstance());
		TabType[] types = { new TabType("Text"), new TabType("Liste"),
				new TabType("Tabelle") };
		typeCombo.setInput(types);
		typeCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			// show column count combobox only if a new table is to be displayed
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				setNumComboVisibitity((IStructuredSelection) typeCombo
						.getSelection());
			}
		});

		// Display the first combo box option ("Text") when going to create a
		// new tab
		final IStructuredSelection selection = new StructuredSelection(types[0]);
		typeCombo.setSelection(selection);
		setNumComboVisibitity(selection);

		// number of columns combo
		numLabel.setText("Columns");
		numCombo.getCombo().setLayoutData(typeComboData);
		numCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (!(element instanceof TabType)) {
					throw new IllegalArgumentException();
				}
				return ((TabType) element).getType();
			}
		});
		numCombo.setContentProvider(ArrayContentProvider.getInstance());
		TabType[] nums = { new TabType("2"), new TabType("3"),
				new TabType("4"), new TabType("5"), new TabType("6"),
				new TabType("7"), new TabType("8") };
		numCombo.setInput(nums);
		numCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				// store the selected column count
				colCountStr = ((TabType) ((IStructuredSelection) numCombo
						.getSelection()).getFirstElement()).getType();
			}
		});

		// Display the first combo box option ("2") when going to create a
		// new table and save it as default
		final ISelection sel = new StructuredSelection(nums[0]);
		numCombo.setSelection(sel);
		colCountStr = nums[0].getType();

		return container;
	}

	/**
	 * Manages the visibility of the number combo box used to select the number
	 * of table columns. The combo box ist only shown when a new table has to be
	 * created.
	 * 
	 * @param sel
	 *            <code>IStructuredSelection</code>, the selection defining the
	 *            tab type.
	 */
	private void setNumComboVisibitity(IStructuredSelection sel) {
		if (sel == null || sel.getFirstElement() == null)
			return;
		String s = ((TabType) sel.getFirstElement()).getType();
		if (s.equals("Tabelle")) {
			numCombo.getCombo().setVisible(true);
			numLabel.setVisible(true);
		} else {
			numCombo.getCombo().setVisible(false);
			numLabel.setVisible(false);
		}
	}

	public String getType() {
		return tabTypeStr;
	}

	public String getColCount() {
		return colCountStr;
	}

	protected void saveInput() {
		super.saveInput();
		tabTypeStr = typeCombo.getCombo().getText();
		colCountStr = numCombo.getCombo().getText();
	}

	@Override
	protected Point getInitialSize() {
		return new Point(450, 210);
	}

	@Override
	public void okPressed() {
		saveInput();
		if (tabTypeStr == null || tabTypeStr.equals("")) {
			MessageDialog md = new MessageDialog(this.getShell(), "", null,
					"Bitte einen Tab-Typ w\u00e4hlen!", SWT.ICON_WARNING,
					new String[] { "OK" }, 0);
			md.open();
			return;
		}
		super.okPressed();
	}
}

class TabType {
	String type;

	public TabType(String type) {
		this.type = type;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}
}
