package de.javabeginners.plugin.notice.ui.elements;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.List;

/**
 * Listen-View-Komponente zur Darstellung in einem Tab
 * 
 * @author Joerg Czeschla, javabeginners.de
 *
 */
public class NoticeList extends NoticeComposite implements NoticeButtonManager {

	/**
	 * the viewer instance
	 */
	private ListViewer viewer;
	/**
	 * the button panel
	 */
	private ButtonComposite buttComp;
	/**
	 * id of this tab
	 */
	private final String id;
	/**
	 * the content provider of this list
	 */
	private ListContentProvider lcp;
	/**
	 * the settings storing the contents of the view tab title an view type are
	 * stored in separate setting sections
	 */
	private IDialogSettings settings;

	public NoticeList(String id, Composite parent, ButtonComposite buttComp,
			IDialogSettings settings, int style) {
		super(parent, id, style);
		this.id = id;
		this.settings = settings;
		this.buttComp = buttComp;
		this.viewer = new ListViewer(this, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		setupUI();
	} // Konstruktor

	/**
	 * create the UI
	 */
	private void setupUI() {
		GridData viewerData = new GridData(GridData.FILL_HORIZONTAL
				| GridData.FILL_VERTICAL);
		viewer.getControl().setLayoutData(viewerData);
		lcp = new ListContentProvider(id, settings);
		viewer.setContentProvider(lcp);
		viewer.setInput(settings.getArray(id));
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			// double click listener
			public void doubleClick(DoubleClickEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				if (selection != null
						&& (selection instanceof IStructuredSelection)) {
					// changes the list value
					change((String) selection.getFirstElement());
				}
			}
		});
		viewer.setLabelProvider(new ILabelProvider() {
			public String getText(Object obj) {
				return (String) obj.toString();
			}

			@Override
			public void addListener(ILabelProviderListener listener) {
			}

			@Override
			public void dispose() {
			}

			@Override
			public boolean isLabelProperty(Object element, String property) {
				return false;
			}

			@Override
			public void removeListener(ILabelProviderListener listener) {
			}

			@Override
			public Image getImage(Object element) {
				return null;
			}
		});
	}

	private void change(String oldValue) {
		InputDialog inputDialog = new InputDialog(getShell(), "Neue Eingabe",
				"Bitte den neuen Wert eingeben", "", null);
		if (inputDialog.open() != Window.OK) {
			return;
		}
		String newValue = inputDialog.getValue();
		lcp.replace(viewer, oldValue, newValue);
		viewer.refresh();
	}

	/**
	 * Deletes the selected list element
	 */
	public void delete() {
		List list = viewer.getList();
		if (list.getItemCount() > 0) {
			String[] sel = list.getSelection();
			if (sel == null || sel.length < 1)
				return;
			String s = sel[0];
			if (s == null)
				return;
			lcp.removeElement(s);
			viewer.refresh();
		}
	}

	/**
	 * Clears the list
	 */
	public void clear() {
		lcp.removeElements();
		viewer.refresh();
	}

	/**
	 * Add an element to the list
	 */
	public void add() {
		List list = viewer.getList();
		int i = list.getSelectionIndex();
		if (i == -1)
			i = list.getItemCount();
		NewListDataInputDialog inputDialog = new NewListDataInputDialog(
				getShell(), "New List Value",
				"Insert new list value");
		if (inputDialog.open() != Window.OK) {
			return;
		}
		// get the String from the dialog...
		String s = inputDialog.getValue();
		// ...store it in the contentProvider...
		((ListContentProvider) viewer.getContentProvider()).addElement(s);
		// and set it to the viewer
		viewer.setInput(s);
	}

	@Override
	public Object getContents() {
		if (viewer instanceof ListViewer) {
			return lcp.getContentAsString();
		}
		return null;
	}

	@Override
	public void setContents(Object obj) {
	}

	@Override
	public Control getControl() {
		return buttComp;
	}

	@Override
	public ISelectionProvider getSelectionProvider() {
		return null;
	}

	public String getID() {
		return id;
	}
}

class ListContentProvider implements IStructuredContentProvider {
	IDialogSettings settings;
	String id;
	ArrayList<String> objects;

	public ListContentProvider(String id, IDialogSettings settings) {
		System.out.println("Create ContentProvider for list '" + id + "'");
		this.id = id;
		this.settings = settings;
		// get the stored values and convert them to an ArrayList
		String[] s = settings.getArray(id);
		if (s != null)
			objects = new ArrayList<String>(Arrays.asList(s));
		else
			objects = new ArrayList<String>();
	}

	public void addElement(Object obj) {
		objects.add((String) obj);
	}

	public void removeElement(Object obj) {
		objects.remove(obj);
	}

	public void removeElements() {
		Object[] obj = objects.toArray();
		for (Object o : obj) {
			objects.remove(o);
		}
		settings.put(id, new String[0]);
	}

	public Object[] getElements(Object obj) {
		return objects.toArray();
	}

	public void dispose() {
		if (objects == null)
			return;
		settings.put(id, getStrings());
	}

	public void replace(Viewer viewer, String oldValue, String newValue) {
		int i = objects.indexOf(oldValue);
		objects.set(i, newValue);
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		String[] stringArray = Arrays.copyOf(objects.toArray(),
				objects.toArray().length, String[].class);
		settings.put(id, stringArray);
		viewer.refresh();
	}

	private String[] getStrings() {
		String[] stringArray = Arrays.copyOf(objects.toArray(),
				objects.toArray().length, String[].class);
		return stringArray;
	}

	public String getContentAsString() {
		String[] strings = getStrings();
		if (strings == null)
			return "";
		StringBuilder builder = new StringBuilder();
		for (String s : strings) {
			builder.append(s + System.getProperty("line.separator"));
		}
		return builder.toString();
	}
}
