package de.javabeginners.plugin.notice.ui.views;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.UUID;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.javabeginners.plugin.notice.ui.Activator;
import de.javabeginners.plugin.notice.ui.elements.NewTabDialog;
import de.javabeginners.plugin.notice.ui.elements.NoticeComposite;
import de.javabeginners.plugin.notice.ui.elements.NoticeCompositeFactory;
import de.javabeginners.plugin.notice.ui.elements.NoticeList;
import de.javabeginners.plugin.notice.ui.elements.NoticeTable;
import de.javabeginners.plugin.notice.ui.elements.NoticeText;
import de.javabeginners.plugin.notice.ui.elements.RenameTabDialog;

/**
 * Main view window of the not.ice plugin. Contains a <code>CTabFolder</code> to
 * display multiple tabs. Each of these may show
 * <dl>
 * <dt>text</dt>
 * <dd>An area showing editable plain text</dd>
 * <dt>list</dt>
 * <dd>A list of any length</dd>
 * <dt>table</dt>
 * <dd>A table of any length with up to eight columns</dd>
 * </dl>
 * 
 * @author J&ouml;rg Czeschla, javabeginners.de
 * 
 *
 */
public class NoticeView extends ViewPart {

	/**
	 * ID of the view as specified by the extension.
	 */
	public static final String ID = "de.javabeginners.plugin.notice.ui.views.NoticeView";
	/**
	 * Default name of the file which is used to save exported data.
	 */
	private static final String EXPORT_FILE_NAME_PREFIX = "notice";
	/**
	 * Key for to store the array with item IDs in the @{@link #settings}
	 */
	private static final String ALL_IDS = "items";
	private static final String DEFAULT_TITLE = "New Tab";
	private static final String DEFAULT_TYPE = "Text";
	private static final String TYPE_SETTINGS = "type";
	private static final String TITLE_SETTINGS = "title";
	public static final String COLCOUNT_SETTINGS = "colCount";

	/**
	 * The NoticeComposite instance shown by a CTabItem. The instance will be of
	 * type <code>NoticeText</code> , <code>NoticeList</code> or
	 * <code>NoticeTable</code>.
	 */
	private NoticeComposite noticeComp;
	/**
	 * Action to clear the tab contents
	 */
	private Action clearAction;
	/**
	 * Action to export the tab contents
	 */
	private Action exportAction;
	/**
	 * Action to create an new tab
	 */
	private Action newTabAction;
	/**
	 * Store the contents of the tabs. Keys are the tab-ids. Read and set in the
	 * view classes
	 */
	private IDialogSettings settings;
	/**
	 * Store the types of the tabs. Keys are the tab-names. Read and set in
	 * <code>NoticeView</code>.
	 */
	private IDialogSettings typeSettings;
	/**
	 * Stores the titles of the tabs.The key is a randomly generated String as
	 * it is done by <code>UUID.randomUUID()</code>.
	 */
	private IDialogSettings titleSettings;
	/**
	 * Stores the number of columns of a table tab
	 */
	private IDialogSettings colCountSettings;
	/**
	 * Contains the tabs of the view
	 */
	private CTabFolder tabFolder;
	/**
	 * Stores the setting of the tab folder.
	 */
	private IMemento memento;

	public NoticeView() {
		settings = Activator.getDefault().getDialogSettings();
		typeSettings = settings.getSection(TYPE_SETTINGS);
		if (typeSettings == null)
			typeSettings = settings.addNewSection(TYPE_SETTINGS);
		titleSettings = settings.getSection(TITLE_SETTINGS);
		if (titleSettings == null)
			titleSettings = settings.addNewSection(TITLE_SETTINGS);
		colCountSettings = settings.getSection(COLCOUNT_SETTINGS);
		if (colCountSettings == null)
			colCountSettings = settings.addNewSection(COLCOUNT_SETTINGS);
	}

	public void init(IViewSite site, IMemento memento) throws PartInitException {
		super.init(site, memento);
		this.memento = memento;
	}

	/**
	 * Saves the IDs of the tabs in an <code>IMemento</code>. This is done
	 * creating an comma separated String of them. This String is split when
	 * it's read.
	 */
	public void saveState(IMemento memento) {
		super.saveState(memento);
		StringBuffer buf = new StringBuffer();
		String[] ids = new String[tabFolder.getItemCount()];
		for (int i = 0; i < ids.length; i++) {
			ids[i] = ((NoticeComposite) tabFolder.getItem(i).getData()).getID();
			buf.append(ids[i]);
			buf.append(',');
		}
		memento.putString(ALL_IDS, buf.toString());
	}

	/**
	 * Initialises the <code>CTabFolder</code> of the view and creates the tabs.
	 * The titles and types of the tabs are read from
	 * <code>IDialogSettings</code>.
	 */
	public void createPartControl(Composite parent) {
		tabFolder = new CTabFolder(parent, SWT.NONE);
		tabFolder.setSimple(false);
		if (memento == null)
			return;
		String value = memento.getString(ALL_IDS);
		if (value == null)
			return;
		String[] ids = value.split(",");
		// creating the tabs
		if (ids == null || ids.length == 0) {
			createTab(createID(), DEFAULT_TITLE, DEFAULT_TYPE,
					NoticeTable.DEFAULT_COLCOUNT);
		} else {
			for (int i = 0; i < ids.length; i++) {
				// get the title and type of the tab from the settings
				String title = titleSettings.get(ids[i]);
				String tabType = typeSettings.get(ids[i]);
				String colCount = colCountSettings.get(ids[i]);
				if (tabType != null)
					createTab(ids[i], title, tabType, colCount);
				else
					createTab(createID(), DEFAULT_TITLE, DEFAULT_TYPE,
							NoticeTable.DEFAULT_COLCOUNT);
			}
		}

		// double click listener for to rename a tab
		tabFolder.addListener(SWT.FOCUSED, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.FOCUSED && tabFolder.getItemCount() > 0) {
					CTabItem item = tabFolder.getItem(tabFolder
							.getSelectionIndex());
					int clickcount = e.count;
					if (clickcount == 2) {
						renameTab(item);
					}
				}
			}
		});

		// Create the help context id for the viewer's control
		PlatformUI
				.getWorkbench()
				.getHelpSystem()
				.setHelp(noticeComp.getControl(),
						"de.javabeginners.plugin.notice.ui.viewer");
		makeActions();
		contributeToActionBars();
	}

	/**
	 * Creates the toolbar of the view
	 */
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalToolBar(bars.getToolBarManager());
	}

	/**
	 * Create the toolbar
	 * 
	 * @param manager
	 *            <code>IToolBarManager</code>, the toolbar manager
	 */
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(newTabAction);
		manager.add(clearAction);
		manager.add(new Separator());
		manager.add(exportAction);
	}

	/**
	 * Create the actions of the view's toolbar.
	 */
	private void makeActions() {
		newTabAction = new Action() {
			public void run() {
				createTab();
			}
		};
		newTabAction.setText("New Tab");
		newTabAction.setToolTipText("New Tab");
		newTabAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_NEW_WIZARD));

		clearAction = new Action() {
			public void run() {
				deleteActItemContents();
			}
		};
		clearAction.setText("Clear all");
		clearAction.setToolTipText("Clear all");
		clearAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_ETOOL_CLEAR));

		exportAction = new Action() {
			public void run() {
				export();
			}
		};
		exportAction.setText("Export...");
		exportAction.setToolTipText("Export...");
		exportAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_ETOOL_SAVE_EDIT));

	}

	/**
	 * Create a file dialog to export the data displayed by the actually visible
	 * tab item.
	 */
	private void export() {
		FileDialog dialog = new FileDialog(super.getSite().getShell(), SWT.SAVE);
		String[] filterNames = new String[] { "Text Files", "All Files (*)" };
		String[] filterExtensions = new String[] { "*.txt;*.xml;*.htm;*.html",
				"*" };
		String filterPath = settings.get("lastOpend");
		filterNames = new String[] { "Text Files", "All Files (*.*)" };
		filterExtensions = new String[] { "*.txt;*.xml;*.htm;*.html;", "*.*" };
		dialog.setFilterNames(filterNames);
		dialog.setFilterExtensions(filterExtensions);
		dialog.setFilterPath(filterPath);
		String tabTitle = tabFolder.getItem(tabFolder.getSelectionIndex())
				.getText().toLowerCase();
		if (!tabTitle.equals("")) {
			tabTitle = "_" + tabTitle;
		}
		dialog.setFileName(EXPORT_FILE_NAME_PREFIX + tabTitle + ".txt");

		File file = null;
		String fileName = dialog.open();
		if (fileName != null) {
			settings.put("lastOpened", fileName);
			file = new File(fileName);
			try (PrintWriter pw = new PrintWriter(new FileWriter(file))) {
				file.createNewFile();
				String s = getActItemContents();
				if (s != null)
					pw.print(s);
			} catch (IOException e) {
				System.err.println("Could not export data.");
				e.printStackTrace();
			}
		}
	}

	public void setFocus() {
		if (noticeComp == null)
			return;
		if (noticeComp.getControl() != null)
			noticeComp.getControl().setFocus();
	}

	/**
	 * 
	 * @return <code>String</code>, a pseudo random ID.
	 */
	private String createID() {
		return UUID.randomUUID().toString();
	}

	/**
	 * Overloading @{@link #createTab(String, String, String, String)}. The method is
	 * called to create a new tab at runtime.
	 */
	public void createTab() {
		NewTabDialog ntd = new NewTabDialog(this.getSite().getShell());
		int open = ntd.open();
		if (open != Window.OK) {
			return;
		}
		String id = createID();
		String titleStr = ntd.getTitle();
		String typeStr = ntd.getType();
		String colCountStr = ntd.getColCount();
		// handle cancel button but allow tabs with empty title
		if (titleStr == null)
			return;
		createTab(id, titleStr, typeStr, colCountStr);
	}

	/**
	 * Creates a new tab, registering the tab with its settings.
	 * 
	 * @param id
	 *            <code>String</code>, id that is used to store the title and
	 *            the type of the tab in the settings.
	 * @param title
	 *            <code>String</code>, the title to be shown on the tab.
	 * @param type
	 *            <code>String</code>, the type of the tab. May be text, list or
	 *            table.
	 * @param colCount
	 *            <code>String</code>, number of columns to create if the tab
	 *            type is a table.
	 */
	private void createTab(String id, String title, String type, String colCount) {
		if (id == null || id.equals(""))
			id = createID();
		System.out.println("About to create a tab with id '" + id + "' named '"
				+ title + "' of type '" + type + "' with contents '"
				+ settings.get(title) + "'.");
		// store the type and title of the new tab
		typeSettings.put(id, type);
		colCountSettings.put(id, colCount);
		titleSettings.put(id, title);
		NoticeCompositeFactory noticeCompFactory = new NoticeCompositeFactory(
				tabFolder, id, settings);
		noticeComp = noticeCompFactory.getNoticeComposite(type);
		if (noticeComp == null)
			return;
		CTabItem item = new CTabItem(tabFolder, SWT.CLOSE);
		item.setText(title);
		item.setData(noticeComp);
		item.setControl(noticeComp.getControl());
		tabFolder.setSelection(item);
		storeAllItemIDs();
	}

	/**
	 * Changes the item title and stores it in the settings. Afterwords the
	 * contents of the item in question is saved with the new name of the item
	 * as key.
	 * 
	 * @param item
	 *            <code>CTabItem</code> the item that has to be renamed
	 */
	private void renameTab(CTabItem item) {
		if (item == null)
			return;
		String id = ((NoticeComposite) item.getData()).getID();
		RenameTabDialog rtd = new RenameTabDialog(this.getSite().getShell());
		int open = rtd.open();
		if (open != Window.OK) {
			return;
		}
		// new title
		String s = rtd.getTitle();
		// if the new title is empty kepp the ols one
		s = s == null ? item.getText() : s;
		item.setText(s);
		// store all ids
		storeAllItemIDs();
		// store the new title
		titleSettings.put(id, s);
	}

	/**
	 * Saves all item titles in an array and stores it in the settings.
	 */
	private void storeAllItemIDs() {
		String[] ids = new String[tabFolder.getItemCount()];
		for (int i = 0; i < ids.length; i++) {
			ids[i] = ((NoticeComposite) tabFolder.getItem(i).getData()).getID();
		}
		settings.put(ALL_IDS, ids);
	}

	/**
	 * Gets the contents of the selected tab as a String. The String is returned
	 * as is if the tab is of type text or list and of type csv, separated by
	 * semicolons by default, if the tab is of type table.<br>
	 * See Also: {@link #export()}
	 * 
	 * @return <code>String</code>, the contents of the selected item or
	 *         <code>null</code> if empty.
	 */
	private String getActItemContents() {
		CTabItem item = tabFolder.getItem(tabFolder.getSelectionIndex());
		String str = null, s = ";";
		if (((NoticeComposite) item.getData()).getContents() instanceof Object[][]) {
			InputDialog inputDialog = new InputDialog(
					null,
					"CSV-Trenner",
					"Bitte ein Zeichen zur Datentrennung f\u00fcr den *.csv-Export eingeben",
					";", null);
			if (inputDialog.open() != Window.OK) {
				return null;
			}
			s = inputDialog.getValue();
			NoticeComposite tab = (NoticeComposite) item.getData();
			Object o = tab.getContents();
			StringBuffer buff = new StringBuffer();
			if (o instanceof String[][]) {
				String[][] sArr = (String[][]) o;
				for (int i = 0; i < sArr.length; i++) {
					for (int j = 0; j < sArr[i].length; j++) {
						buff.append(sArr[i][j]);
						if (j < sArr[i].length - 1)
							buff.append(s);
					}
					buff.append(System.getProperty("line.separator"));
				}
			}
			str = buff.toString();
		} else if (((NoticeComposite) item.getData()).getContents() instanceof Object[]) {
			InputDialog inputDialog = new InputDialog(null, "CSV-Trenner",
					"Bitte ein Zeichen eingeben", "", null);
			if (inputDialog.open() != Window.OK) {
				return null;
			}
			s = inputDialog.getValue();
			Object[] o = (Object[]) ((NoticeComposite) item.getData())
					.getContents();
			for (Object obj : o) {
				str += obj.toString() + s;
			}
			str = str.substring(0, str.length() - 1);
		} else {
			Object o = (Object) ((NoticeComposite) item.getData())
					.getContents();
			str = (String) o;
		}
		// Maybe null
		return str;
	}

	/**
	 * Deletes all data in a <code>NoticeText</code>. see comment:
	 * {@link #getActItemContents()}
	 * 
	 */
	private void clearText() {
		CTabItem item = tabFolder.getItem(tabFolder.getSelectionIndex());
		if (item.getData() instanceof NoticeComposite)
			((NoticeComposite) item.getData()).setContents("");
	}

	/**
	 * Deletes all data in a <code>NoticeList</code>.
	 */
	private void clearList() {
		CTabItem item = tabFolder.getItem(tabFolder.getSelectionIndex());
		if (item.getData() instanceof NoticeList)
			((NoticeList) item.getData()).clear();
	}

	/**
	 * Deletes all data in a <code>NoticeTable</code>.
	 */
	private void clearTable() {
		CTabItem item = tabFolder.getItem(tabFolder.getSelectionIndex());
		if (item.getData() instanceof NoticeTable)
			((NoticeTable) item.getData()).clear();
	}

	/**
	 * Deletes the contents of the selected tab
	 * 
	 */
	private void deleteActItemContents() {
		CTabItem item = tabFolder.getItem(tabFolder.getSelectionIndex());
		String id = ((NoticeComposite) item.getData()).getID();
		settings.put(id, "");
		if (item.getData() instanceof NoticeText) {
			clearText();
		} else if (item.getData() instanceof NoticeList) {
			clearList();
		} else if (item.getData() instanceof NoticeTable) {
			clearTable();
		}
	}
}